#!/usr/bin/env python
# slide-into-place-ani.py
# Created by TT
# Just thought this would be interesting, to create animation that slides into place with different layers from different angles and from different angles
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1.0: Initial release http://gimpchat.com/viewtopic.php?f=9&t=20614&sid=77a9f061a556bd7aca930c874d98fb6e
# Rel 1.1: Sat Nov 18, 2023 Added option to animation in reverse as well so that it can be endless loop. 

from gimpfu import *
#import random
import math


def ease_in_out(t):
    """Ease-in-out function: acceleration until halfway, then deceleration"""
    return 2*t*t if t < 0.5 else 1 - pow(-2*t + 2, 2) / 2
def python_slide_into_place_ani(image,layer,slideframes,framearray,anglearray,distancearray,fadearray,outro):
	# pdb.gimp_image_undo_group_start(image)
	# pdb.gimp_context_push()
	#YOUR CODE BEGINS=======================
	slideframes = int(slideframes)
	
	new_image = pdb.gimp_image_new(image.width,image.height,RGB)
	pdb.gimp_display_new(new_image)
	
	linkedlayers = []
	visible = []
	positions = []
	for l in image.layers:
		if pdb.gimp_item_get_linked(l) == TRUE:
			linkedlayers.append(l)
			visible.append(pdb.gimp_item_get_visible(l))
			positions.append(pdb.gimp_image_get_item_position(image,l))
	linkedlayers.reverse()
	visible.reverse()
	positions.reverse()

	#hide all linked layers so it doesn't affect our animation layers
	for l in linkedlayers:
		pdb.gimp_item_set_visible(l,FALSE)

	framearray = [int(x) for x in framearray.split(',')]
	anglearray = [int(x) for x in anglearray.split(',')]
	distancearray = [int(x) for x in distancearray.split(',')]
	fadearray = [int(x) for x in fadearray.split(',')]
	framearray.reverse()
	anglearray.reverse()
	distancearray.reverse()
	fadearray.reverse()
	frames = max(framearray)+slideframes
	insert_position = 0;
	for j in range(0,frames):
		removelayers = []
		#pdb.gimp_message("NEW FRAME")
		for i in range(0,len(linkedlayers)):
			linkedlayer = linkedlayers[i]
			thisframe = j-framearray[i]
			if thisframe >= 0 and thisframe <= slideframes:
				t = float(thisframe)/slideframes
			elif thisframe < 0:
				t = 0
			else:
				t = 1;
			#pdb.gimp_message(t)	
			mindist = 0; maxdist = distancearray[i];
			dist = maxdist + (mindist-maxdist) * ease_in_out(t)
			opacity = 0 + (1.0) * ease_in_out(t)
			#text_layer = pdb.gimp_text_fontname(image,None,0,0,word,-1,TRUE,size,0,fontname)
			new_layer = pdb.gimp_layer_new_from_drawable(linkedlayer,image)

			pdb.gimp_image_insert_layer(image,new_layer,None,positions[i])
			pdb.gimp_item_set_linked(new_layer,FALSE)
			pdb.gimp_item_set_visible(new_layer,TRUE)
			angle = anglearray[i]
			rad = angle/360.0*2*math.pi
			ix = math.cos(rad) * dist
			iy = math.sin(rad) * dist

			offsetx,offsety = pdb.gimp_drawable_offsets(new_layer)
			pdb.gimp_layer_set_offsets(new_layer,offsetx+ix,offsety+iy)
			if (fadearray[i] == 1):
				pdb.gimp_layer_set_opacity(new_layer,opacity * 100)
			removelayers.append(new_layer) #add it so we can remove it when we're done

		if (int(outro)==0):	
			visible_layer = pdb.gimp_layer_new_from_visible(image,new_image,'frame')
			pdb.gimp_image_insert_layer(new_image,visible_layer,None,insert_position)
		else:
			visible_layer = pdb.gimp_layer_new_from_visible(image,new_image,'frame')
			pdb.gimp_image_insert_layer(new_image,visible_layer,None,insert_position)
			visible_layer = pdb.gimp_layer_new_from_visible(image,new_image,'frame')
			pdb.gimp_image_insert_layer(new_image,visible_layer,None,insert_position)
			insert_position += 1

		for rl in removelayers:
			pdb.gimp_image_remove_layer(image,rl)
	#set it back to whatever visibility it was	
	for i in range(0,len(linkedlayers)):
		l = linkedlayers[i]
		v = visible[i]
		pdb.gimp_item_set_visible(l,v)
	
	pdb.gimp_displays_flush()

    #return

register(
	"python_fu_slide_into_place_ani",
	"Slide Into Place Animation (for linked layers).\n Ver.1.1",
	"Slide Into Place Animation",
	"TT",
	"TT",
	"Nov 5, 2023",
	"<Image>/Filters/Animation/Animators/Slide Into Place...",
	"*",  
	[ 
	(PF_INT, "frames", "Frames per Slide:", 10),
	(PF_STRING, "framearray", "Start Sliding-In Frame Array (1st is for frame):", "1,5,10,20,1"),
	(PF_STRING, "anglearray", "Sliding-In From Angle Array:", "0,45,90,180,270"),
	(PF_STRING, "distancearray","Sliding-In From Distance Array:","100,200,300,400,150"),
	(PF_STRING, "fadearray","Sliding-In Fade Array(1:fade,0:no-fade:","1,1,1,1,0"),
	(PF_TOGGLE, "outro",   "Animate Outro (in reverse as well):", 0), # initially True, checked.  Alias PF_BOOL
	],
	[],
	python_slide_into_place_ani)

main()

